#include "stdafx.h"
#include "testDigestUpload.h"
#include "util.h"

size_t BaseUpload::Read ( char *data, size_t size )
{
  if ( !m_IsInitialized || m_emptyUpload || m_istream.eof() )
    return 0;

  m_istream.read(data, size);
  // when bufsize > readsize eof AND failbit is set!!
  if( !m_istream.eof() && (m_istream.fail() || m_istream.bad()) )
  {
    m_IsInitialized = false;
    throw std::exception("Failed to read for uploading");
  }

  return m_istream.gcount();
}

CFileUpload::CFileUpload ( const std::wstring &localPath )
: BaseUpload ( m_ifstream )
{
  m_fileName = localPath;
  if(m_fileName == L"")
  {
    m_emptyUpload = true;
    return;
  }
  m_ifstream.open(localPath.c_str(), std::ios::binary | std::ios::in );
  if(!m_ifstream.is_open())
  {
    m_IsInitialized = false;
    throw std::exception("Failed to open file for uploading");
  }

  // get filesize
  long begin = m_ifstream.tellg();
  m_ifstream.seekg (0, std::ios::end);
  long end = m_ifstream.tellg();
  m_ifstream.clear();
  m_ifstream.seekg(0);
  m_size = end-begin;
}

CFileUpload::~CFileUpload () 
{ 
  m_ifstream.close();
}

class CCurlSlistHandle
{
public:
  CCurlSlistHandle () : m_handle ( NULL )
  {
  }

  ~CCurlSlistHandle ()
  {
    if ( m_handle )
      curl_slist_free_all ( m_handle );
  }

  void Add ( const char *pitem )
  {
    if ( pitem )
      m_handle = curl_slist_append ( m_handle, pitem ); // NOTE: string is copied by libcurl
  }

  void Add ( const std::string &item )
  {
    Add ( item.c_str () );
  }

  operator const curl_slist * () const { return m_handle; }
  operator curl_slist * () { return m_handle; }

private:
  curl_slist *m_handle;
};

BaseUpload *upload;

static size_t read_callback(void *ptr, size_t size, size_t nmemb, void *userp)
{
  return upload->Read ( (char *)ptr, size * nmemb );
}

int testDigestReadCallback ( char *URL )
{
  CURL *c;
  CURLM *m;
  int res = 0;
  int running=1;
  int MULTI_PERFORM_HANG_TIMEOUT = 50000;
  struct timeval mp_start;
  char mp_timedout = FALSE;

  CCurlSlistHandle *m_Headers = new CCurlSlistHandle ();

  upload = new CFileUpload(L"c:/test.xml"); // SET HERE YOUR XML FILE

  if (curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
    fprintf(stderr, "curl_global_init() failed\n");
    return -1;
  }

  if ((c = curl_easy_init()) == NULL) {
    fprintf(stderr, "curl_easy_init() failed\n");
    curl_global_cleanup();
    return -1;
  }

  curl_easy_setopt ( c, CURLOPT_URL, URL );
  curl_easy_setopt(c, CURLOPT_READFUNCTION, read_callback );
  curl_easy_setopt ( c, CURLOPT_READDATA, upload );

  curl_easy_setopt ( c, CURLOPT_POST, 1);
  curl_easy_setopt ( c, CURLOPT_POSTFIELDSIZE, upload->Size() );

  m_Headers->Add ( "application/x-www-form-urlencoded" );
  m_Headers->Add ( "Expect: 100-continue" );
  curl_easy_setopt ( c, CURLOPT_HTTPHEADER, *m_Headers );

  curl_easy_setopt ( c, CURLOPT_HTTPAUTH, CURLAUTH_DIGEST );
  std::string login("doesntmatter@example.com");
  std::string pass("mypass");
  std::string authString = login + ":" + pass;

  curl_easy_setopt(c, CURLOPT_USERPWD, authString.c_str() );

  if ((m = curl_multi_init()) == NULL) {
    fprintf(stderr, "curl_multi_init() failed\n");
    curl_easy_cleanup(c);
    curl_global_cleanup();
    return -1;
  }

  if ((res = (int)curl_multi_add_handle(m, c)) != CURLM_OK) {
    fprintf(stderr, "curl_multi_add_handle() failed, "
      "with code %d\n", res);
    curl_multi_cleanup(m);
    curl_easy_cleanup(c);
    curl_global_cleanup();
    return -1;
  }

  mp_timedout = FALSE;
  mp_start = tutil_tvnow();

  while (running) {
    res = (int)curl_multi_perform(m, &running);
    if (tutil_tvdiff(tutil_tvnow(), mp_start) > 
      MULTI_PERFORM_HANG_TIMEOUT) {
        mp_timedout = TRUE;
        break;
    }
    if (running <= 0) {
      fprintf(stderr, "nothing left running.\n");
      break;
    }
  }

  if (mp_timedout) {
    if (mp_timedout) fprintf(stderr, "mp_timedout\n");
    fprintf(stderr, "ABORTING TEST, since it seems "
      "that it would have run forever.\n");
    res = -2;
  }

  curl_multi_remove_handle(m, c);
  curl_easy_cleanup(c);
  curl_multi_cleanup(m);
  curl_global_cleanup();

  delete m_Headers;
  delete upload;

  return res;
}

int testDigest (char *URL)
{
  CURL *c;
  CURLM *m;
  int res = 0;
  int running=1;
  int MULTI_PERFORM_HANG_TIMEOUT = 50000;
  struct timeval mp_start;
  char mp_timedout = FALSE;

  CCurlSlistHandle *m_Headers = new CCurlSlistHandle ();

  BaseUpload *upload = new CFileUpload(L"c:/test.xml"); // SET HERE YOUR XML FILE

  if (curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
    fprintf(stderr, "curl_global_init() failed\n");
    return -1;
  }

  if ((c = curl_easy_init()) == NULL) {
    fprintf(stderr, "curl_easy_init() failed\n");
    curl_global_cleanup();
    return -1;
  }

  curl_easy_setopt ( c, CURLOPT_URL, URL );
  
  int numRead = upload->Size();
  char *charBuf = new char [numRead];
  upload->Read ( charBuf, numRead );
  curl_easy_setopt ( c, CURLOPT_POSTFIELDS, charBuf );
  curl_easy_setopt ( c, CURLOPT_POSTFIELDSIZE, numRead );

  m_Headers->Add ( "application/x-www-form-urlencoded" );
  curl_easy_setopt ( c, CURLOPT_HTTPHEADER, *m_Headers );

  curl_easy_setopt ( c, CURLOPT_HTTPAUTH, CURLAUTH_DIGEST );
  std::string login("doesntmatter@example.com");
  std::string pass("mypass");
  std::string authString = login + ":" + pass;

  curl_easy_setopt(c, CURLOPT_USERPWD, authString.c_str() );

  if ((m = curl_multi_init()) == NULL) {
    fprintf(stderr, "curl_multi_init() failed\n");
    curl_easy_cleanup(c);
    curl_global_cleanup();
    return -1;
  }

  if ((res = (int)curl_multi_add_handle(m, c)) != CURLM_OK) {
    fprintf(stderr, "curl_multi_add_handle() failed, "
      "with code %d\n", res);
    curl_multi_cleanup(m);
    curl_easy_cleanup(c);
    curl_global_cleanup();
    return -1;
  }

  mp_timedout = FALSE;
  mp_start = tutil_tvnow();

  while (running) {
    res = (int)curl_multi_perform(m, &running);
    if (tutil_tvdiff(tutil_tvnow(), mp_start) > 
      MULTI_PERFORM_HANG_TIMEOUT) {
        mp_timedout = TRUE;
        break;
    }
    if (running <= 0) {
      fprintf(stderr, "nothing left running.\n");
      break;
    }
  }

  if (mp_timedout) {
    if (mp_timedout) fprintf(stderr, "mp_timedout\n");
    fprintf(stderr, "ABORTING TEST, since it seems "
      "that it would have run forever.\n");
    res = -2;
  }

  curl_multi_remove_handle(m, c);
  curl_easy_cleanup(c);
  curl_multi_cleanup(m);
  curl_global_cleanup();

  delete m_Headers;
  delete upload;
  delete[] charBuf;

  return res;
}